@@ -1,7 +1,7 @@
 #
 # field.rb
 #
-#   Copyright (c) 1998-2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+#   Copyright (c) 1998-2001 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -17,7 +17,7 @@ require 'tmail/encode'
 
 module TMail
 
-  MSGID = /<[^\@>]+\@[^>\@]+>/o
+  MSGID = /<[^\@>]+\@[^>\@]+>/
 
   ZONESTR_TABLE = {
     'jst' =>   9 * 60,
@@ -86,20 +86,20 @@ module TMail
         m[1] == '-' ? -sec : sec
       else
         unless min = ZONESTR_TABLE[ str.downcase ] then
-          raise ParseError, "wrong timezone format '#{str}'"
+          raise MailSyntaxError, "wrong timezone format '#{str}'"
         end
         min * 60
       end
     end
 
-    def time2str( time )
-      ret = time.strftime( "%a, #{time.mday} %b %Y %X " )
+    def time2str( tm )
+      ret = tm.strftime( "%a, #{tm.mday} %b %Y %X " )
 
-      # Time#gmtime changes self!!!
-      tm = Time.at( time.to_i )
       # [ruby-list:7928]
-      offset = tm.to_i - Time.mktime( *tm.gmtime.to_a[0, 6].reverse ).to_i
-      ret << '%+.2d%.2d' % (offset / 60).divmod( 60 )
+      gmt = Time.at(tm.to_i)
+      gmt.gmtime
+      offset = tm.to_i - Time.local( *gmt.to_a[0,6].reverse ).to_i
+      ret << sprintf( '%+.2d%.2d', *(offset / 60).divmod(60) )
 
       ret
     end
@@ -424,8 +424,8 @@ module TMail
 
       begin
         init_real
-        Mailp.parse( @body, self, nil )
-      rescue ParseError, ScanError
+        Parser.parse( @body, self, nil )
+      rescue MailSyntaxError, ScanError
         if not pre and ::TMail.encoded?( @body ) then
           pre = @body
           @body = HFdecoder.decode( pre )
@@ -462,10 +462,7 @@ module TMail
 
     parse_on_rw
 
-    header_attr :date, Time, R | W | NODUP, nil, <<S
-      arg = Time.at( arg.to_i )
-      arg.localtime
-S
+    header_attr :date, Time, R | W | NODUP
 
     redefine_eql
     empty_able
@@ -579,6 +576,7 @@ S
       route     ==  other.route  and
       phrase    ==  other.phrase
     end
+
     alias == eql?
 
     def dup
@@ -618,12 +616,12 @@ S
       super arg ? arg.dup : []
     end
 
-
     attr :name, true
     
     def eql?( oth )
       super( oth ) and self.name == oth.name
     end
+
     alias == eql?
 
     def each_address
@@ -763,12 +761,24 @@ S
       end
     end
 
+    def msgids
+      ret = []
+      each_msgid {|i| ret.push i }
+      ret
+    end
+
     def each_phrase
       refs.each do |i|
         yield i unless ::TMail.msgid? i
       end
     end
 
+    def phrases
+      ret = []
+      each_phrase {|i| ret.push i }
+      ret
+    end
+
 
     private
 
@@ -838,8 +848,8 @@ S
     header_attr :by,    String, R | W
     header_attr :via,   String, R | W
     header_attr :with,  Array,  R | NODUP
-    header_attr :msgid, String, R | W
-    header_attr :for_,  String, R | W,        [:ford, :for_domain]
+    header_attr :_id,   String, R | W,        [:msgid]
+    header_attr :_for,  String, R | W,        [:for_]
     header_attr :date,  Time,   R | W | NODUP
 
     redefine_eql
@@ -849,7 +859,7 @@ S
     private
 
     def init
-      @from = @by = @via = @with = @msgid = @for_ = nil
+      @from = @by = @via = @with = @_id = @_for = nil
       @with = []
       @date = nil
     end
@@ -857,10 +867,10 @@ S
     Tag = %w( from by via with id for )
 
     def do_accept( stra )
-      val = [ @from, @by, @via ] + @with + [ @msgid, @for_ ]
+      val = [ @from, @by, @via ] + @with + [ @_id, @_for ]
       val.compact!
       c   = [ @from ? 1 : 0, @by ? 1 : 0, @via ? 1 : 0,
-              @with.size, @msgid ? 1 : 0, @for_ ? 1 : 0 ]
+              @with.size, @_id ? 1 : 0, @_for ? 1 : 0 ]
 
       i = 0
       c.each_with_index do |count, idx|
