@@ -1,7 +1,7 @@
 #
 # tmail.rb
 #
-#   Copyright (c) 1998-2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+#   Copyright (c) 1998-2001 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -9,6 +9,7 @@
 #
 
 require 'nkf'
+require 'socket'
 
 require 'amstd/to_s'
 require 'amstd/bug'
@@ -20,11 +21,46 @@ require 'tmail/loader'
 require 'tmail/encode'
 
 
-class MailError < StandardError; end
-
-
 module TMail
 
+  class MailError < StandardError; end
+  class MailSyntaxError < MailError; end
+
+
+  class << self
+
+    def new_boundary
+      'mimepart_' + random_tag
+    end
+
+    alias boundary new_boundary
+
+    def new_msgid( fqdn = nil )
+      domain = (fqdn && "#{fqdn}.tmail") || ::Socket.gethostname
+      "<#{random_tag}@#{domain}>"
+    end
+
+    alias msgid new_msgid
+
+    private
+
+    def random_tag
+      @uniq += 1
+      t = Time.now
+      sprintf( '%x%x_%x%x%d%x',
+               t.to_i,
+               t.tv_usec,
+               $$,
+               Thread.current.id,
+               @uniq,
+               rand(255) )
+    end
+
+  end
+
+  @uniq = 0
+
+
   class Mail
 
     class << self
@@ -35,25 +71,8 @@ module TMail
 
       alias loadfrom load_from
 
-
-      def boundary
-        'mimepart_' + random_tag
-      end
-
-      def msgid
-        if host = ENV['HOSTNAME'] then
-          "<#{random_tag}@#{host}>"
-        else
-          "<#{random_tag}@tmail.on.ruby>"
-        end
-      end
-
-      private
-
-      def random_tag
-        ret = ('%x' % Time.now.strftime('%Y%m%d%H%M%S').to_i) + '_'
-        1.upto(8){ srand ; ret << ('%x' % rand(255)) }
-        ret
+      def parse( str )
+        new StringPort.new( str )
       end
 
     end
@@ -73,6 +92,10 @@ module TMail
 
     attr :port
 
+    def inspect
+      "\#<#{type} port=#{@port.inspect} bodyport=#{@body_port.inspect}>"
+    end
+
 
     def encoded( eol = "\r\n", charset = 'j', ret = '', sep = '' )
       en = HFencoder.new( ret, eol, charset )
@@ -86,17 +109,13 @@ module TMail
       ret
     end
 
-    alias to_s encoded
-    alias inspect decoded
-    # def inspect
-    #   "#<TMail::Mail: port=#{@port.inspect}, body=#{@body_port.inspect}>"
-    # end
+    alias to_s decoded
 
     def accept( stra, eol = "\n", ret = '', sep = '' )
-      multipart = ! parts.empty?    # DO NOT USE @parts
+      multipart_p = ! parts.empty?    # DO NOT USE @parts
 
-      if multipart then
-        bound = Mail.boundary
+      if multipart_p then
+        bound = ::TMail.new_boundary
         h = self['content-type']
         if h then
           h.params['boundary'] = bound
@@ -114,10 +133,9 @@ module TMail
         end
       end
 
-      if multipart then
-        boundary = "#{eol}--#{bound}#{eol}"
+      if multipart_p then
         parts.each do |tm|
-          ret << boundary
+          ret << "#{eol}--#{bound}#{eol}"
           tm.accept( stra, eol, ret, sep )
         end
         ret << "#{eol}--#{bound}--#{eol}"
@@ -150,7 +168,7 @@ module TMail
     end
 
 
-    def []( key, initbody = nil, &block )
+    def fetch( key, initbody = nil, &block )
       dkey = key.downcase
       ret = @header[ dkey ]
 
@@ -167,7 +185,7 @@ module TMail
       ret
     end
 
-    def []=( key, val )
+    def store( key, val )
       dkey = key.downcase
 
       if val.nil? then
@@ -190,13 +208,6 @@ module TMail
         val = mkhf( key, val.to_s )
       end
 
-      do_store dkey, val
-    end
-
-    alias fetch []
-    alias store []=
-
-    def do_store( dkey, val )
       if USE_ARRAY.include? dkey then
         if arr = @header[dkey] then
           arr.push val
@@ -207,7 +218,12 @@ module TMail
         @header[dkey] = val
       end
     end
-    private :do_store
+
+    def []( key )
+      @header[ key.downcase ]
+    end
+
+    alias []= store
 
 
     def each_header( &block )
@@ -352,40 +368,50 @@ module TMail
 
     def parse_header
       fname = fbody = nil
+      unixfrom = nil
       errlog = []
 
-      src = @stream = @port.ropen
+      begin
+        @stream = src = @port.ropen
 
-      while line = src.gets do     # no each !
-        case line
-        when /\A[ \t]/             # continue from prev line
-          unless fbody then
-            errlog.push 'mail is began by space or tab'
-            next
+        while line = src.gets do     # no each !
+          case line
+          when /\A[ \t]/             # continue from prev line
+            unless fbody then
+              errlog.push 'mail is began by space or tab'
+              next
+            end
+            line.strip!
+            fbody << ' ' << line
+            # fbody << line
+
+          when /\A([^\: \t]+):\s*/   # new header line
+            add_hf fname, fbody if fbody
+            fname = $1
+            fbody = $'
+            # fbody.strip!
+
+          when /\A\-*\s*\z/          # end of header
+            add_hf fname, fbody if fbody
+            break
+
+          when /\AFrom (\S+)/
+            unixfrom = $1
+
+          else
+            errlog.push "wrong mail header: '#{line.inspect}'"
           end
-          line.strip!
-          fbody << ' ' << line
-          # fbody << line
-
-        when /\A([^\: \t]+):\s*/   # new header line
-          add_hf fname, fbody if fbody
-          fname = $1
-          fbody = $'
-          # fbody.strip!
-
-        when /\A\-*\s*\z/          # end of header
-          add_hf fname, fbody if fbody
-          break
-
-        else
-          errlog.push "wrong mail header: '#{line.inspect}'"
         end
+      ensure
+        src.stop if src
       end
       unless errlog.empty? then
-        raise ParseError, "\n" + errlog.join("\n")
+        raise MailSyntaxError, "\n" + errlog.join("\n")
       end
 
-      src.stop
+      if unixfrom then
+        add_hf 'Return-Path', "<#{unixfrom}>" unless @header['return-path']
+      end
     end
 
     def add_hf( fname, fbody )
@@ -515,9 +541,7 @@ module TMail
       end
 
       @body_port = ports.shift
-
-      ports.filter {|p| Mail.new( p ) }
-      @parts = ports
+      @parts = ports.collect {|p| type.new p }
     end
 
     def tmp_port( n = 0 )
@@ -538,8 +562,7 @@ module TMail
       def tmail_attr( mname, hname, part )
         module_eval %-
           def #{mname.id2name}( default = nil )
-            header = self['#{hname}']
-            if header then
+            if header = @header['#{hname}'] then
               header.#{part}
             else
               default
@@ -555,7 +578,7 @@ module TMail
 
     def date=( t )
       store 'Date', ''
-      self['date'].date = t
+      @header['date'].date = t
     end
 
     tmail_attr :to_addrs, 'to', 'addrs'
@@ -575,50 +598,61 @@ module TMail
       store 'To', addr
     end
 
-    def destinations
-      ret = []
-      %w( to cc bcc ).each do |hname|
-        if hed = self[ hname ] then
-          hed.addrs.each {|i| ret.push i.address }
-        end
-      end
-      ret
+    def to_addrs=( arr )
+      store 'To', ''
+      @header['to'].addrs.replace arr
     end
 
 
     tmail_attr :from_addrs, 'from', 'addrs'
 
-    def from( dflt = '' )
-      arr = from_addrs
-      if arr and arr[0] then
-        arr[0].addr
-      else
-        dflt
-      end
+    def from_addrs=( arr )
+      store 'From', ''
+      @header['from'].addrs.replace arr
     end
 
-    def from_phrase( dflt = '' )
-      arr = from_addrs
-      if arr and arr[0] then
-        arr[0].phrase
+    def from_addr( default = nil )
+      if arr = from_addrs(nil) then
+        arr[0] || default
       else
-        dflt
+        default
       end
     end
 
     def from_address( default = nil )
-      if head = self['From'] then
-        if a = head.addrs[0] then
-          return a.address
-        end
+      if a = from_addr(nil) then
+        a.address
+      else
+        default
       end
-      default
     end
 
-    def from=( addr )
+    alias from from_address
+
+    def from_address=( addr )
       store 'From', addr
     end
 
+    alias from= from_address=
+
+    def from_phrase( default = nil )
+      if a = from_addr(nil) then
+        a.phrase
+      else
+        default
+      end
+    end
+
+
+    tmail_attr :reply_to_addrs, 'reply-to', 'addrs'
+
+
+    tmail_attr :sender, 'sender', 'addr'
+
+    def sender=( addr )
+      store 'Sender', addr
+    end
+
 
     tmail_attr :subject, 'subject', 'body'
 
@@ -628,6 +662,7 @@ module TMail
       else
         store 'Subject', str
       end
+      str
     end
 
 
@@ -635,6 +670,23 @@ module TMail
 
     def msgid=( str )
       store 'Message-ID', str
+      str
+    end
+
+
+    tmail_attr :references, 'references', 'msgids'
+
+    def references=( arr )
+      store 'References', ''
+      @header['references'].refs.replace arr
+    end
+
+
+    tmail_attr :in_reply_to, 'in-reply-to', 'msgids'
+
+    def in_reply_to=( arr )
+      store 'In-Reply-To', ''
+      @header['in-reply-to'].refs.replace arr
     end
 
 
@@ -651,40 +703,41 @@ module TMail
       else
         store 'Mime-Version', m
       end
+      m
     end
 
 
     tmail_attr :main_type, 'content-type', 'main'
     tmail_attr :sub_type, 'content-type', 'sub'
 
-    def content_type=( str )
-      if head = self['content-type'] then
-        head.main, header.sub = str.split('/', 2)
+    def set_content_type( str, sub = nil, param = nil )
+      if sub then
+        main, sub = str, sub
       else
-        store 'Content-Type', str
+        main, sub = str.split('/', 2)
       end
-    end
-        
-    def set_content_type( main, sub, param = nil )
       if head = self['content-type'] then
         head.main = main
         head.sub  = sub
       else
-        store 'Content-Type', main + '/' + sub
-        if param then
-          self['content-type'].params.replace param
-        end
+        store 'Content-Type', "#{main}/#{sub}"
       end
+      self['content-type'].params.replace param if param
+
+      str
     end
 
+    alias content_type= set_content_type
+
     tmail_attr :charset, 'content-type', "params['charset']"
 
     def charset=( str )
-      if hed = self[ 'content-type' ] then
-        hed.params.store 'charset', str
+      if head = self[ 'content-type' ] then
+        head.params['charset'] = str
       else
-        store "text/plain ; charset=#{str}"
+        store "text/plain; charset=#{str}"
       end
+      str
     end
 
 
@@ -692,25 +745,70 @@ module TMail
 
     def encoding=( str )
       store 'Content-Transfer-Encoding', str
+      str
     end
 
 
+    tmail_attr :disposition, 'content-disposition', 'disposition'
+
+    def disposition=( str )
+      store 'Content-Disposition', str
+      str
+    end
+
+    def set_content_disposition( str, params = nil )
+      if f = @header['content-disposition'] then
+        f.disposition = str
+      else
+        f = store( 'Content-Disposition', str )
+      end
+      f.params.replace params if params
+    end
+
+
+    ###
+    ### utils
+    ###
+
+    def destinations
+      ret = []
+      %w( to cc bcc ).each do |nm|
+        if head = @header[nm] then
+          head.addrs.each {|i| ret.push i.address }
+        end
+      end
+      ret
+    end
+
     def each_destination( &block )
       destinations.each do |i|
         if Address === i then
           yield i
         else
-          i.each &block
+          i.each( &block )
         end
       end
     end
 
     alias each_dest each_destination
 
-    def multipart?
-      main_type('').downcase == 'multipart'
+
+    def reply_addresses
+      reply_to_addrs(nil) or from_addrs(nil)
     end
 
-  end   # class TMail::Mail
+    def error_reply_addresses
+      if s = sender(nil) then
+        [s]
+      else
+        from_addrs(nil)
+      end
+    end
+
+    def multipart?
+      /downcase/i === main_type('')
+    end
+
+  end   # class Mail
 
 end   # module TMail
